(* ----------------------------------------------------------------- *)
(* Turbo-Pascal Unit, Vers. 2.4

   Unterroutinen fr Datentransfer (ASCII oder binr) ber eine
   serielle Schnittstelle.
   Die neue Version TPCOMX untersttzt mehrere serielle Schnittstellen
   gleichzeitig. Der Datenempfang wird interrupt-gepuffert, wenn einer der
   folgenden speicheresidenten Schnittstellentreiber installiert ist:
   von DOS oder in AUTOEXEC.BAT:
          COMDRV.COM   (fr eine COM-Schnittstelle)
          COMnDRV.COM  (fr n COM-Schnittstellen)
     oder PC32DRV.COM  (fr Meilhaus PC31-Interface mit 8 Schnittst.)
   (erfordert mind. Treiber-Version 3.3)

   ACHTUNG: Die interne Numerierung der Schnittstellen (Variable "ComNr")
   ======== erfolgt von 0 an (d.h. COM1 hat ComNr = 0, etc.)

   Status: (ComInit und ComStat)
           Bit 15 = 0
               14 = Transmitter shift reg. empty
               13 = Transmitter holding register empty
               12 = Break interrupt
               11 = Framing error
               10 = Parity error
                9 = Overrun error
                8 = Data received (buffer not empty)
                7 = Received line signal detect (RLSD)
                6 = Ring indicator (RI)
                5 = Data set ready (DSR)
                4 = Clear to send (CTS)
                3 = Delta received lin signal detect
                2 = Trailing edge ring indicator
                1 = Delta data set ready
                0 = Delta clear to send

   J. Rathlev, Apr. 1990
      Rev. 2.3, Feb. 1995 (opt. Warteschleife bei Ausgabe)
      Rev. 2.4, Jun. 1995 (mit Zeilenzhler fr Eingabepuffer) *)

unit TPComX;

(* ======================================================================= *)
interface

uses Dos, Crt, NStrings, TPSub;

const
  NoPar = 'N';
  EvenPar = 'E';
  OddPar = 'O';

  ParityModes = [Nopar,EvenPar,OddPar];

  (* nachfolgende Variable kann vom Hauptprogramm verndert werden *)
  ComOutWait : boolean = false;
  (* false: bei Ausgabe ber COM nicht warten bis Schnittstelle bereit
            (Zeichen gehen verloren)
     true : bei Ausgabe warten bis Schnittstelle bereit *)

type
  ComModeRec = record
    baud    : longint;
    par     : char;
    Len,Stb : byte;
    end;

(* ----------------------------------------------------------------------- *)
(* serielle Schnittstelle initialisieren und Modem einschalten *)
procedure ComInit (ComNr       : word;
                   Baud        : longint;
                   Par         : char;
                   Len,Stb     : word;
               var Status      : word);
(* ComNr = Nummer der ser. Schnittstelle (0,1,..)
   Baud  = Baud Rate (110 ... 38400)
   Par   = Paritt (N,E,O)
   Len   = Anzahl der Datenbits (7,8)
   Stb   = Anzahl der Stopbits (1,2)
   Status = Line und Modem Status *)

(* ----------------------------------------------------------------------- *)
(* ser. Schnittstelle ComNr mit Mode-String ComPar initialisieren (siehe auch
   DOS-Befehl "MODE" *)
procedure SetComMode (ComNr      : integer;
                      ComPar     : ComModeRec;
                      var Err    : boolean;
                      var Status : word);

(* ----------------------------------------------------------------------- *)
(* Schnittstellenbuffer lschen und Modem einschalten *)
procedure ComClr (ComNr : word);
procedure ComOn (ComNr : word);
(* ComNr = Nummer der ser. Schnittstelle (0,1,..) *)

(* ----------------------------------------------------------------------- *)
(* Modem ausschalten *)
procedure ComOff (ComNr : word);
(* ComNr = Nummer der ser. Schnittstelle (0,1,..) *)

(* ----------------------------------------------------------------------- *)
(* Status der ser. Schnittstelle abfragen *)
function ComStat (ComNr : word) : word;
(* ComNr = Nummer der ser. Schnittstelle (0,1,..) *)

(* LCR-Status der ser. Schnittstelle abfragen *)
function ComLCR (ComNr : word) : byte;
(* ComNr = Nummer der ser. Schnittstelle (0,1,..) *)

(* ----------------------------------------------------------------------- *)
(* Buffer Count der ser. Schnittstelle abfragen *)
function ComRcvBuf (ComNr : word) : word;
(* ComNr = Nummer der ser. Schnittstelle (0,1,..) *)

(* Zeilenzahl in Eingabepuffer *)
function ComRcvLines (ComNr : word) : integer;

(* Output Buffer Count der ser. Schnittstelle abfragen *)
function ComSndBuf (ComNr : word) : word;

(* Output Buffer Lnge der ser. Schnittstelle abfragen *)
function ComSndLen (ComNr : word) : word;

(* ----------------------------------------------------------------------- *)
(* Schnittstellentreiber-Identifizierungsbyte abfragen *)
function ComId (ComNr   : word) : word;

(* Name des geladenen Schnittstellentreibers *)
function ComDrvName (ComNr : word) : string;
(* ComNr = Nummer der ser. Schnittstelle (0,1,..) *)

(* ----------------------------------------------------------------------- *)
(* Warten auf Input ber die ser. Schnittstelle *)
function ComRec (ComNr   : word;
                 TimeOut : integer) : boolean;
(* ComNr = Nummer der ser. Schnittstelle (0,1,..)
   Timeout = Zeit in 1/10 Sek., die gewartet wird. Wird in
             dieser Zeit ein Zeichen empfangen, ist ComRec=true *)

(* ----------------------------------------------------------------------- *)
(* Zeichen von ser. Schnittstelle einlesen *)
function ComRead (ComNr       : word;
                  var ErrStat : byte) : char;
(* ComNr = Nummer der ser. Schnittstelle (0,1,..)
   ErrStat = Fehlerstatus : Bit 1 = Overrun Error
                            Bit 2 = Parity Error
                            Bit 3 = Framing Error
                            Bit 4 = Break Interrupt *)

(* ----------------------------------------------------------------------- *)
(* Zeichen an ser. Schnittstelle ausgeben *)
procedure ComWrite (ComNr   : word;
                    Zeichen : char);
(* ComNr = Nummer der ser. Schnittstelle (0,1,..)
   Zeichen = auszugebendes Zeichen *)

(* ----------------------------------------------------------------------- *)
(* String von ser. Schnittstelle einlesen *)
function ComRdStr (ComNr       : word;
                   var ErrStat : byte) : string;
(* ComNr = Nummer der ser. Schnittstelle (0,1,..)
   ErrStat = Fehlerstatus : Bit 1 = Overrun Error
                            Bit 2 = Parity Error
                            Bit 3 = Framing Error
                            Bit 4 = Break Interrupt
   ComRdStr = String bis Carriage Return
   untersttzt XOn/XOff-Protokoll *)

(* ----------------------------------------------------------------------- *)
(* String an ser. Schnittstelle ausgeben *)
procedure ComWrStr (ComNr : word;
                    Zeile : string);
(* ComNr = Nummer der ser. Schnittstelle (0,1,..)
   Zeile = zu schreibender String (ohne CR/LF) *)

(* ----------------------------------------------------------------------- *)
(* CR/LF an ser. Schnittstelle ausgeben *)
procedure ComWrCrLf (ComNr : word);
(* ComNr = Nummer der ser. Schnittstelle (0,1,..) *)

(* ----------------------------------------------------------------------- *)
(* Integer-Zahl an ser. Schnittstelle ausgeben *)
procedure ComWrInt (ComNr : word;
                    x,n   : integer);
(* ComNr = Nummer der ser. Schnittstelle (0,1,..)
   x     = Integerzahl
   n     = Anzahl der Stellen *)

(* ----------------------------------------------------------------------- *)
(* Real-Zahl an ser. Schnittstelle ausgeben *)
procedure ComWrReal (ComNr : word;
                     x     : double;
                     n,d   : integer);
(* ComNr = Nummer der ser. Schnittstelle (0,1,..)
   x     = Realzahl
   n     = Anzahl der Stellen
   d     = Anzahl der Stellen hinter dem Dezimalpunkt *)

(* ======================================================================= *)
implementation
const
  strLoad = ' loaded for COM';
  strNoDrv = 'No driver loaded for COM';

(* ----------------------------------------------------------------------- *)
(* serielle Schnittstelle initialisieren und Modem einschalten*)
procedure ComInit (ComNr       : word;
                   Baud        : longint;
                   Par         : char;
                   Len,Stb     : word;
               var Status      : word);
(* ComNr = Nummer der ser. Schnittstelle (0,1,..)
   Baud  = Baud Rate (110 ... 38400)
   Par   = Paritt (N,E,O)
   Len   = Anzahl der Datenbits (7,8)
   Stb   = Anzahl der Stopbits (1,2)
   Status = Line und Modem Status (siehe auch ComStat) *)
var
   reg : registers;
   i,k :integer;
begin
  i:=Baud div 10;
  case i of    (* Baud Rate *)
        5 : k:=0;
        7 : k:=32;
       11 : k:=2;
       13 : k:=64;
       15 : k:=34;
       30 : k:=66;
       60 : k:=98;
      120 : k:=130;
      180 : k:=96;
      240 : k:=162;
      480 : k:=194;
      960 : k:=226;
     1920 : k:=128;
     3840 : k:=160;
     5760 : k:=192;
    11520 : k:=224;
       else k:=194;
       end;
     case UpCase(Par) of     (* Paritt *)
       NoPar   : i:=0;
       OddPar  : i:=1;
       EvenPar : i:=3;
       else i:=0;
       end;
     k:=k+8*i;
     case Stb of     (* Stop Bits *)
       1 : i:=0;
       2 : i:=1;
       else i:=0;
       end;
     k:=k+4*i;
     case Len of     (* Anzahl der Bits *)
       7 : i:=0;
       8 : i:=1;
       else i:=1;
       end;
     k:=k+i; reg.ax:=k; reg.dx:=ComNr;
     intr ($14,reg);
     Status:=reg.ax;
     end;

(* ----------------------------------------------------------------------- *)
(* ser. Schnittstelle CNr mit Mode-String CPar initialisieren (siehe auch
   DOS-Befehl "MODE" *)
procedure SetComMode (ComNr      : integer;
                      ComPar     : ComModeRec;
                      var Err    : boolean;
                      var Status : word);
begin
  with ComPar do ComInit (ComNr,Baud,Par,Len,Stb,status);
  end;

(* ----------------------------------------------------------------------- *)
(* Schnittstellenbuffer lschen und Modem einschalten *)
procedure ComClr (ComNr : word);
(* ComNr = Nummer der ser. Schnittstelle (0,1,..) *)
var
   reg : registers;
begin
   reg.ax:=$400; reg.dx:=ComNr;
   intr ($14,reg);
   end;

(* ----------------------------------------------------------------------- *)
(* Schnittstellenbuffer lschen und Modem einschalten (identisch mit ComClr *)
procedure ComOn (ComNr : word);
(* ComNr = Nummer der ser. Schnittstelle (0,1,..) *)
var
   reg : registers;
begin
   reg.ax:=$400; reg.dx:=ComNr;
   intr ($14,reg);
   end;

(* ----------------------------------------------------------------------- *)
(* Modem ausschalten *)
procedure ComOff (ComNr : word);
(* ComNr = Nummer der ser. Schnittstelle (0,1,..) *)
var
   reg : registers;
begin
   reg.ax:=$500; reg.dx:=ComNr;
   intr ($14,reg);
   end;

(* ----------------------------------------------------------------------- *)
(* Status der ser. Schnittstelle abfragen *)
function ComStat (ComNr : word) : word;
(* ComNr = Nummer der ser. Schnittstelle (0,1,..)
   Status: Bit 15 = 0
               14 = Transmitter shiftv reg. empty
               13 = Transmitter holding register empty
               12 = Break interrupt
               11 = Framing error
               10 = Parity error
                9 = Overrun error
                8 = Data received (buffer not empty)
                7 = Received line signal detect (RLSD)
                6 = Ring indicator (RI)
                5 = Data set ready (DSR)
                4 = Clear to send (CTS)
                3 = Delta received lin signal detect
                2 = Trailing edge ring indicator
                1 = Delta data set ready
                0 = Delta clear to send *)
var
   reg : registers;
begin
   reg.ax:=$300; reg.dx:=ComNr;
   intr ($14,reg);
   ComStat:=reg.ax;
   end;

(* ----------------------------------------------------------------------- *)
(* Line Control Register der ser. Schnittstelle abfragen
   siehe Datenblatt 8250 *)
function ComLCR (ComNr : word) : byte;
(* ComNr = Nummer der ser. Schnittstelle (0,1,..) *)
var
   reg : registers;
begin
   reg.ax:=$600; reg.dx:=ComNr;
   intr ($14,reg);
   ComLCR:=reg.al;
   end;

(* ----------------------------------------------------------------------- *)
(* Input Buffer Count der ser. Schnittstelle abfragen *)
function ComRcvBuf (ComNr : word) : word;
(* ComNr = Nummer der ser. Schnittstelle (0,1,..) *)
var
   reg : registers;
begin
   reg.ax:=$700; reg.dx:=ComNr;
   intr ($14,reg);
   ComRcvBuf:=reg.ax;
   end;

(* ----------------------------------------------------------------------- *)
(* Zeilenzahl in Eingabepuffer *)
function ComRcvLines (ComNr : word) : integer;
(* ComNr = Nummer der ser. Schnittstelle (0,1,..) *)
var
   reg : registers;
begin
   with reg do begin
     ax:=$700; dx:=ComNr; cx:=$FFFF;
     intr ($14,reg);
     ComRcvLines:=cx;
     end;
   end;

(* ----------------------------------------------------------------------- *)
(* Output Buffer Count der ser. Schnittstelle abfragen *)
function ComSndBuf (ComNr : word) : word;
(* ComNr = Nummer der ser. Schnittstelle (0,1,..) *)
var
   reg : registers;
begin
   reg.ax:=$701; reg.dx:=ComNr;
   intr ($14,reg);
   ComSndBuf:=reg.ax;
   end;

(* ----------------------------------------------------------------------- *)
(* Output Buffer Lnge der ser. Schnittstelle abfragen *)
function ComSndLen (ComNr : word) : word;
(* ComNr = Nummer der ser. Schnittstelle (0,1,..) *)
var
   reg : registers;
begin
   reg.ax:=$701; reg.dx:=ComNr;
   intr ($14,reg);
   ComSndLen:=reg.dx;
   end;

(* ----------------------------------------------------------------------- *)
(* Schnittstellentreiber-Identifizierungsbyte abfragen *)
function ComId (ComNr   : word) : word;
(* ComNr = Nummer der ser. Schnittstelle (0,1,..)
   Ergebnis = 0     : kein Treiber installiert
            = $Fxvv : Treiber Typ x, Vers. vv installiert
                      (siehe auch DrvName) *)
var
  reg : registers;
begin
  with reg do begin
    ax:=$800; dx:=ComNr;
    intr ($14,reg);
    if al=0 then ah:=0;
    ComID:=ax;
    end;
  end;

(* ----------------------------------------------------------------------- *)
(* Name des geladenen Schnittstellentreibers *)
function ComDrvName (ComNr : word) : string;
(* ComNr = Nummer der ser. Schnittstelle (0,1,..) *)
var
  n   : byte;
  reg : registers;
  p   : pointer;
begin
  with reg do begin
    ax:=$800; dx:=ComNr;
    intr ($14,reg);
    if al=0 then ComDrvName:=strNoDrv+StrInt(succ(ComNr),1)
    else begin
      p:=ptr(es,bx);
      ComDrvName:=StrPas(p)+strLoad+StrInt(succ(ComNr),1)+' - Port '+
                  HexWord(dx)+' - IRQ '+RemSp(StrInt(cl,2));
      end;
    end;
  end;

(* ----------------------------------------------------------------------- *)
(* Warten auf Input ber die ser. Schnittstelle *)
function ComRec (ComNr   : word;
                 TimeOut : integer) : boolean;
(* ComNr = Nummer der ser. Schnittstelle (0,1,..)
   Timeout = Zeit in 1/10 Sek., die gewartet wird. Wird in
             dieser Zeit ein Zeichen empfangen, ist ComRec=true *)
begin
   while ((ComStat(ComNr) and $100)=0) and (TimeOut>=0) do begin
     dec(TimeOut); delay (100);
     end;
   if TimeOut<0 then ComRec:=false else ComRec:=true;
   end;

(* ----------------------------------------------------------------------- *)
(* Zeichen von ser. Schnittstelle einlesen *)
function ComRead (ComNr       : word;
                  var ErrStat : byte) : char;
(* ComNr = Nummer der ser. Schnittstelle (0,1,..)
   ErrStat = Fehlerstatus : Bit 1 = Overrun Error
                            Bit 2 = Parity Error
                            Bit 3 = Framing Error
                            Bit 4 = Break Interrupt *)
var
   reg : registers;
begin
   repeat
     reg.ax:=$200; reg.dx:=ComNr;
     intr($14,reg);
     until (reg.ah and $80)=0;
     ComRead:=chr(reg.al);
     ErrStat:=reg.ah;
     end;

(* ----------------------------------------------------------------------- *)
(* Zeichen an ser. Schnittstelle ausgeben
   ComOutWait (siehe ComInit) = true : warten, wenn Buffer voll
                           = false: Zeichen nicht senden wenn Buffer voll *)
procedure ComWrite (ComNr   : word;
                    Zeichen : char);
(* ComNr = Nummer der ser. Schnittstelle (0,1,..)
   Zeichen = auszugebendes Zeichen *)
var
   reg : registers;
   ok  : boolean;
begin
  with reg do repeat
    ax:=$100+ord(Zeichen); dx:=ComNr;
    intr ($14,reg);
    ok:=(not ComOutWait) or (ah and $80=0);
    if not ok then delay (100);
    until ok;
  end;

(* ----------------------------------------------------------------------- *)
(* String von ser. Schnittstelle einlesen mit Ctrl-Q/S-Prokoll*)
function ComRdStr (ComNr       : word;
                   var ErrStat : byte) : string;
(* ComNr = Nummer der ser. Schnittstelle (0,1,..)
   ErrStat = Fehlerstatus : Bit 1 = Overrun Error
                            Bit 2 = Parity Error
                            Bit 3 = Framing Error
                            Bit 4 = Break Interrupt
   ComRdStr = String bis Carriage Return
   untersttzt XOn/XOff-Protokoll *)
var
  z : char;
  s : string;
begin
  s:=''; (*ComWrite(ComNr,^Q);*)
  repeat
    z:=ComRead(ComNr,ErrStat);
    if z>#31 then s:=s+z;
    until (z=#13) or not ComRec(ComNr,10) or (ErrStat<>0);
  (*ComWrite (ComNr,^S);*) ComRdStr:=s;
  end;

(* ----------------------------------------------------------------------- *)
(* String an ser. Schnittstelle ausgeben *)
procedure ComWrStr (ComNr : word;
                    Zeile : string);
(* ComNr = Nummer der ser. Schnittstelle (0,1,..)
   Zeile = zu schreibender String (ohne CR/LF) *)
var
  i : integer;
  l : byte absolute Zeile;
begin
  for i:=1 to l do ComWrite (ComNr,Zeile[i]);
  end;

(* ----------------------------------------------------------------------- *)
(* CR/LF an ser. Schnittstelle ausgeben *)
procedure ComWrCrLf (ComNr : word);
(* ComNr = Nummer der ser. Schnittstelle (0,1,..) *)
begin
  ComWrite (ComNr,#13); ComWrite (ComNr,#10);
  end;

(* ----------------------------------------------------------------------- *)
(* Integer-Zahl an ser. Schnittstelle ausgeben *)
procedure ComWrInt (ComNr : word;
                    x,n   : integer);
(* ComNr = Nummer der ser. Schnittstelle (0,1,..)
   x     = Integerzahl
   n     = Anzahl der Stellen *)
var
  number : string [10];
begin
  str (x:n,number); ComWrStr (ComNr,number);
  end;

(* ----------------------------------------------------------------------- *)
(* Real-Zahl an ser. Schnittstelle ausgeben *)
procedure ComWrReal (ComNr : word;
                     x     : double;
                     n,d   : integer);
(* ComNr = Nummer der ser. Schnittstelle (0,1,..)
   x     = Realzahl
   n     = Anzahl der Stellen
   d     = Anzahl der Stellen hinter dem Dezimalpunkt *)
var
  number : string [20];
begin
  str (x:n:d,number); ComWrStr (ComNr,number);
  end;

(* ----------------------------------------------------------------------- *)
end.
